<?php
/*
Plugin Name: WP Post Encode
Plugin URI: http://darrinb.com/notes/2010/wp-post-encode/
Description: Easily include raw code  in posts.  Based off of "bb_encodeit" used in bbPress comment system.
Version: 1.0
Author: Darrin Boutote
Author URI: http://darrinb.com
*/
/*
Copyright 2010  Darrin Boutote  (contact : http://darrinb.com/hello/)
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


// Init function to insert button options into wp_options Table/
function dbdb_post_encode_activate() {
    if ( '' == get_option( 'dbdb_post_encode') ) {
        $name = 'dbdb_post_encode';
        $value = array();
        $value['buttons'][0]['text']  = 'encode';
        $value['buttons'][0]['start'] = '<!--encode-->';
        $value['buttons'][0]['end']   = '<!--/encode-->';
        $autoload = 'yes';
        add_option($name, $value, $autoload);
    }
}

// Init function to delete button options from wp_options Table
function dbdb_post_encode_deactivate() {
    delete_option( 'dbdb_post_encode' );
}


// Load buttons only if user is on "post.php", "page.php", "post-new.php", or "page-new.php"
if ( strpos( $_SERVER['REQUEST_URI'], 'post.php' ) || strpos( $_SERVER['REQUEST_URI'], 'post-new.php' ) || strpos( $_SERVER['REQUEST_URI'], 'page-new.php' ) || strpos( $_SERVER['REQUEST_URI'], 'page.php' ) ) {

    add_action( 'admin_footer', 'dbdb_post_encode_buttons' );

    // adds buttons to the Toolbar in the HTML editor
    function dbdb_post_encode_buttons() {

        $o = get_option( 'dbdb_post_encode' );

        // If there is
        if ( count( $o['buttons'] ) > 0 ) {
            echo <<<EOT
            <script type="text/javascript">
                <!--
                    if (dbencToolbar = document.getElementById("ed_toolbar")) {
                        var dbencNr, dbencBut, dbencStart, dbencEnd;
EOT;
                        for ($i = 0; $i < count($o['buttons']); $i++) {
                            $b = $o['buttons'][$i];
                            $txt = html_entity_decode(stripslashes($b['txt']), ENT_COMPAT, get_option('blog_charset'));
                            $text = stripslashes($b['text']);
                            $b['text'] = stripslashes($b['text']);
                            $start = preg_replace('![\n\r]+!', "\\n", $b['start']);
                            $start = str_replace("'", "\'", $start);
                            $end = preg_replace('![\n\r]+!', "\\n", $b['end']);
                            $end = str_replace("'", "\'", $end);
                            echo <<<EOT
                            dbencStart = '{$start}';
                            dbencEnd = '{$end}';
                            dbencNr = edButtons.length;
                            edButtons[dbencNr] = new edButton('ed_dbenc{$i}','{$b['txt']}',dbencStart, dbencEnd,'');
                            var dbencBut = dbencToolbar.lastChild;
                            while (dbencBut.nodeType != 1) {
                                dbencBut = dbencBut.previousSibling;
                            }
                            dbencBut = dbencBut.cloneNode(true);
                            dbencToolbar.appendChild(dbencBut);
                            dbencBut.value = '{$b['text']}';
                            dbencBut.title = dbencNr;
                            dbencBut.onclick = function () {edInsertTag(edCanvas, parseInt(this.title));}
                            dbencBut.id = "ed_dbenc{$i}";
EOT;
                        }
                    echo <<<EOT
                    }
                //-->
            </script>
EOT;
        }
    }
}

// Triggers the buttons' install function upon plugin activation
if ( function_exists( 'register_activation_hook' ) ) {
    register_activation_hook( __FILE__, 'dbdb_post_encode_activate' );
}
// Triggers the buttons' uninstall function upon plugin deactivation
if ( function_exists('register_deactivation_hook') ) {
    register_deactivation_hook(__FILE__, 'dbdb_post_encode_deactivate');
}

// Encode WP default quicktags
function dbdb_quick_encode($content_text) {
    $charset = get_bloginfo('charset');
    $replaced_text = preg_replace('#(<!--encode-->)(.*?)(<!--/encode-->)#isme', "'$1'.str_replace(array('<!--nextpage-->', '<!--more-->'), array('&lt;!--nextpage--&gt;', '&lt;!--more--&gt;'), '$2').'$3'", $content_text);
    foreach($replaced_text as $k => $v ) {
        $encoded_text[$k] = str_replace(array('\"'),array('"'), $v);
    }
    return $encoded_text;
};
add_filter('wp_insert_post_data','dbdb_quick_encode', 1, 1);


function dbdb_post_encode($text) {
    $text = str_replace(array("\r\n", "\r"), "\n", $text);  // replace any carriage returns and/or new lines with a new line.
    $text = preg_replace_callback("#(<!--encode-->)(.*?)(<!--/encode-->)#is", 'dbdb_code_encode', $text);
    return $text;
};
add_filter('the_content','dbdb_post_encode', 1, 1);

function dbdb_code_encode( $matches ) {
    $charset = get_bloginfo('charset');
    $text = trim($matches[2]);
    $text = str_replace(array('&lt;!--nextpage--&gt;', '&lt;!--more--&gt;'), array('<!--nextpage-->', '<!--more-->'), $text); // quicktag filtering
    $text = htmlspecialchars($text, ENT_QUOTES, $charset);  // encode html chars.
    $text = str_replace('[','&#91;', $text);                // shortcode filtering
    $text = str_replace(array("\r\n", "\r"), "\n", $text);  // replace any carriage returns and/or new lines with a new line.
    $text = preg_replace("#\n\n\n+#", "\n\n", $text);       // replace any multiple new lines with 2 new lines.
    return $text;
};
?>